/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.common;

import java.text.Collator;
import java.util.Arrays;
import java.util.Set;

/**
 * Utility class that contains general purpose methods
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
public final class Utils
{
	/**
	 * Returns a signed decimal value of the given String, or 0 if the given String is not a number
	 * 
	 * @param string Number as a string
	 * @return Signed decimal integer value represented by the argument
	 */
	public static int getNumber( String string )
	{
		int number = 0;
		
		try
		{
			number = Integer.parseInt( string );
		}
		catch( NumberFormatException nfe )
		{
			//nothing - 0 will be returned
		}

		return number;
	}
	
	/**
	 * Returns a String object representing the given integer value
	 * 
	 * @param value Integer value
	 * @return String object of the argument in base 10
	 */
	public static String getString( int value )
	{
		return Integer.toString( value );
	}

	/**
	 * Returns a String object representing the given long value
	 * 
	 * @param value Long value
	 * @return String object of the argument in base 10
	 */
	public static String getString( long value )
	{
		return Long.toString( value );
	}

	/**
	 * Returns String object representing the given <code>NumericalEnum</code> value
	 * 
	 * @param numericalEnum NumericalEnum enum
	 * @return String object representing the given argument
	 */
	public static String getEnumValue( NumericalEnum numericalEnum )
	{
		int value = 0;

		if( numericalEnum != null )
		{
			value = numericalEnum.getValue();
		}

		return getString( value );
	}	

	/**
     * Validates that an object is not null
     *
     * @param varName Name of variable to be included with exception
     * @param value Value to check
     * @return Value passed in
     * @throws NullPointerException if value is null
     */
    public static <T extends Object> T notNull( String varName, T value ) throws NullPointerException 
    {
        if( value == null )
        {
            throw new NullPointerException( varName + " cannot be null" );
        }

        return value;
    }

    /**
     * Returns whether the given String is null or empty after trimming white spaces
     * 
     * @param string String object
     * @return true if the given String is null or is still empty after trimming white spaces
     */
    public static boolean isEmpty( String string )
    {
    	return string == null || string.trim().isEmpty();
    }

	/**
	 * Converts Set of Strings to a case-insensitive array of Strings
	 * 
	 * @param set Set containing strings
	 * @return Case-insensitive sorted array of Strings
     * @throws NullPointerException if set is null
	 */
	public static String[] getSortedCaseInsensitiveArray( Set<String> set )
	{
		notNull( "set", set );

		String array[] = set.toArray( new String[0] );

		Arrays.sort( array, Collator.getInstance() );

		return array;
	}
}