/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.game.scan;

import info.msxlaunchers.openmsx.common.FileTypeUtils;
import info.msxlaunchers.openmsx.common.HashUtils;
import info.msxlaunchers.openmsx.common.IOUtils;
import info.msxlaunchers.openmsx.common.Utils;
import info.msxlaunchers.openmsx.game.RepositoryGame;
import info.msxlaunchers.openmsx.game.repository.RepositoryData;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.extra.ExtraData;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataGetter;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseMaxBackupReachedException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNullNameException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameWithNullNameException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import com.google.inject.Inject;

/**
 * Implementation for physical media scanning for games supported by openMSX
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
class FileScanner implements Scanner
{
	private final GamePersister gamePersister;
	private final RepositoryData repositoryData;
	private final ExtraDataGetter extraDataGetter;

	private boolean interrupted;

	private Set<Game> processedGames = null;
	private Set<String> processedGameNames = null;
	private Set<String> processedGameSha1Codes = null;
	private Map<String,RepositoryGame> sha1ToRepositoryGameMap = null;
	private Map<String,ExtraData> extraDataMap = null;

	private boolean traverseSubDirectories;
	private String machine = null;
	private boolean searchROM = false;
	private boolean searchDisk = false;
	private boolean searchTape = false;
	private boolean searchLaserdisc = false;
	private boolean getNameFromOpenMSXDatabase;

	private final String NAME_COLLISION_SEPARATOR = "__";
	private final String MSX_GENERATION_URL = "http://www.generation-msx.nl/msxdb/softwareinfo/";

	private final String EXTENSION_ROM_IDE = "ide";

	@Inject
	FileScanner( GamePersister gamePersister, RepositoryData repositoryData, ExtraDataGetter extraDataGetter )
	{
		this.gamePersister = Utils.notNull( "gamePersister", gamePersister );
		this.repositoryData = Utils.notNull( "repositoryData", repositoryData );
		this.extraDataGetter = Utils.notNull( "extraDataGetter", extraDataGetter );

		//the following flag can be set from a different thread to stop
		interrupted = false;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.game.scan.Scanner#scan(java.lang.String[], boolean, java.lang.String, boolean, boolean, java.lang.String, boolean, boolean, boolean, boolean, boolean, boolean)
	 */
	@Override
	public int scan( String[] paths,
			boolean traverseSubDirectories,
			String database,
			boolean newDatabase,
			boolean append,
			String machine,
			boolean searchROM,
			boolean searchDisk,
			boolean searchTape,
			boolean searchLaserdisc,
			boolean getNameFromOpenMSXDatabase,
			boolean backupDatabase)
		throws GameWithNullNameException, DatabaseMaxBackupReachedException, DatabaseAlreadyExistsException,
				DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		interrupted = false;

		this.traverseSubDirectories = traverseSubDirectories;
		this.machine = machine;
		this.searchROM = searchROM;
		this.searchDisk = searchDisk;
		this.searchTape = searchTape;
		this.searchLaserdisc = searchLaserdisc;
		this.getNameFromOpenMSXDatabase = getNameFromOpenMSXDatabase;

		processedGames = new HashSet<Game>();
		processedGameNames = new HashSet<String>();
		processedGameSha1Codes = new HashSet<String>();

		//append to or create new database
		if( newDatabase )
		{
			gamePersister.createDatabase( database );
			//TODO: what if this db exists?
		}
		else
		{
			if( backupDatabase )
			{
				gamePersister.backupDatabase( database );
			}

			if( !append )
			{
				gamePersister.recreateDatabase( database );
			}

			Set<Game> savedGames = gamePersister.getGames( database );

			for( Game processedGame: savedGames )
			{
				processedGameNames.add( processedGame.getName() );
				processedGameSha1Codes.add( processedGame.getSha1Code() );
			}
		}

		if( getNameFromOpenMSXDatabase )
		{
			sha1ToRepositoryGameMap = repositoryData.getRepositoryInfo();
		}

		//load the extra data map
		extraDataMap = extraDataGetter.getExtraData();

		//start the scanning
		int totalFound = 0;
		for( String path: paths )
		{
			if( interrupted )
			{
				break;
			}

			totalFound += traverse( new File( path ) );
		}

		try
		{
			gamePersister.saveGames( processedGames, database );
		}
		catch( GameAlreadyExistsException ex )
		{
			//TODO: what to do? just skip for now
		}

		return totalFound;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.game.scan.Scanner#interrupt()
	 */
	public void interrupt()
	{
		interrupted = true;
	}

	private int traverse( File path )
	{
		int totalFound = 0;

		File[] fileList = path.listFiles();
		
		if( fileList != null )
		{
			for( File file: fileList )
			{
				if( interrupted )
				{
					break;
				}

				if ( traverseSubDirectories && file.isDirectory() )
				{
					totalFound += traverse( file );
				}
				else
				{
					totalFound += processFile( file );
				}
			}
		}

		return totalFound;
	}

	private int processFile( File file )
	{
		int added = 0;

		if( FileTypeUtils.isROM( file ) ||
				FileTypeUtils.isDisk( file ) ||
				FileTypeUtils.isTape( file ) ||
				FileTypeUtils.isLaserdisc( file ) )
		{
			String sha1Code = HashUtils.getSHA1Code( file );
			added = processPotentialGame( file, file.getAbsolutePath(), file.length(), sha1Code );
		}
		else if( FileTypeUtils.isZIP( file ) )
		{
			added = processZipFile( file );
		}

		return added;
	}

	private int processPotentialGame( File file, String fileNameToUse, long fileSize, String sha1Code )
    {
		int added = 0;

		if( file != null )
		{
			if( searchROM && FileTypeUtils.isROM( file ) )
			{
				added = addToProcessedGames( getGameName( file, sha1Code ),
						fileNameToUse,
						null,
						null,
						null,
						null,
						null,
						sha1Code,
						fileSize);
			}
			else if( searchDisk && FileTypeUtils.isDisk( file ) )
			{
				if( fileSize <= FileTypeUtils.MAX_DISK_FILE_SIZE )
				{
					added = addToProcessedGames( getGameName( file, sha1Code ),
							null,
							fileNameToUse,
							null,
							null,
							null,
							null,
							sha1Code,
							fileSize);
				}
				else
				{
					added = addToProcessedGames( getGameName( file, sha1Code ),
							null,
							null,
							fileNameToUse,
							EXTENSION_ROM_IDE,
							null,
							null,
							sha1Code,
							fileSize);
				}					
			}
			else if( searchTape && FileTypeUtils.isTape( file ) )
			{
				added = addToProcessedGames( getGameName( file, sha1Code ),
						null,
						null,
						null,
						null,
						fileNameToUse,
						null,
						sha1Code,
						fileSize);
			}
			else if( searchLaserdisc && FileTypeUtils.isLaserdisc( file ) )
			{
				added = addToProcessedGames( getGameName( file, sha1Code ),
						null,
						null,
						null,
						null,
						null,
						fileNameToUse,
						sha1Code,
						fileSize);
			}
		}

		return added;
    }
	
	private int processZipFile( File zipFile )
	{
		int added = 0;

		ZipFile zip = null;
		InputStream inputStream = null;

		try
		{
			zip = new ZipFile( zipFile );

			ZipEntry firstZipEntry = zip.entries().nextElement();

			inputStream = zip.getInputStream( firstZipEntry );
			String sha1Code = HashUtils.getSHA1Code( inputStream );

			added = processPotentialGame( new File( firstZipEntry.getName() ),
					zipFile.getAbsolutePath(),
					firstZipEntry.getSize(),
					sha1Code );
		}
		catch( ZipException e )
		{
			//invalid ZIP file - skip and return null
		}
		catch( IOException e )
		{
			//shouldn't happen here
		}
		finally
		{
			IOUtils.forceClose( inputStream );
			IOUtils.forceClose( zip );
		}

		return added;
	}

	private int addToProcessedGames( String name,
										String rom,
										String disk,
										String harddisk,
										String extensionRom,
										String tape,
										String laserdisc,
										String sha1Code,
										long fileSize )
	{
		int added = 0;

		if( !processedGameSha1Codes.contains( sha1Code ) )
		{
			ExtraData extraData = extraDataMap.get( sha1Code );
			int msxGenID = 0;
			boolean isMSX = false;
			boolean isMSX2 = false;
			boolean isMSX2Plus = false;
			boolean isTurboR = false;
			boolean isPSG = false;
			boolean isSCC = false;
			boolean isSCCI = false;
			boolean isPCM = false;
			boolean isMSXMUSIC = false;
			boolean isMSXAUDIO = false;
			boolean isMoonsound = false;
			boolean isMIDI = false;
			String screenshotSuffix = null;
			String msxGenerationURL = null;
			if( extraData != null )
			{
				msxGenID = extraData.getMSXGenerationsID();
				if( msxGenID > 0 )
				{
					msxGenerationURL = MSX_GENERATION_URL + msxGenID;
				}
				isMSX = extraData.isMSX();
				isMSX2 = extraData.isMSX();
				isMSX2Plus = extraData.isMSX();
				isTurboR = extraData.isMSX();
				isPSG = extraData.isPSG();
				isSCC = extraData.isSCC();
				isSCCI = extraData.isSCCI();
				isPCM = extraData.isPCM();
				isMSXMUSIC = extraData.isMSXMUSIC();
				isMSXAUDIO = extraData.isMSXAUDIO();
				isMoonsound = extraData.isMoonsound();
				isMIDI = extraData.isMIDI();
				screenshotSuffix = extraData.getSuffix();
			}

			name = adjustedNameIfNecessary( name );
			processedGames.add( Game.name( name )
					.romA( rom )
					.diskA( disk )
					.harddisk( harddisk )
					.extensionRom( extensionRom )
					.tape( tape )
					.laserdisc( laserdisc )
					.machine( machine )
					.sha1Code( sha1Code )
					.size( fileSize )
					.msxGenID( msxGenID )
					.isMSX( isMSX )
					.isMSX2( isMSX2 )
					.isMSX2Plus( isMSX2Plus )
					.isTurboR( isTurboR )
					.isPSG( isPSG )
					.isSCC( isSCC )
					.isSCCI( isSCCI )
					.isPCM( isPCM )
					.isMSXMUSIC( isMSXMUSIC )
					.isMSXAUDIO( isMSXAUDIO )
					.isMoonsound( isMoonsound )
					.isMIDI( isMIDI )
					.screenshotSuffix( screenshotSuffix )
					.info( msxGenerationURL )
					.build() );

			processedGameNames.add( name );
			processedGameSha1Codes.add( sha1Code );
			
			added = 1;
		}

		return added;
	}

	private String adjustedNameIfNecessary( String name )
	{
		StringBuilder nameBuffer = new StringBuilder( name );
		int counter = 0;

		while( processedGameNames.contains( nameBuffer.toString() ) )
		{
			nameBuffer = new StringBuilder( name ).append( NAME_COLLISION_SEPARATOR ).append( ++counter );
		}

		return nameBuffer.toString();
	}

	private String getGameName( File file, String sha1Code )
	{
		String name = null;

		if( getNameFromOpenMSXDatabase )
		{
			RepositoryGame repositoryGame = sha1ToRepositoryGameMap.get( sha1Code );
			if( repositoryGame != null )
			{
				name = repositoryGame.getTitle();
				if( name == null )
				{
					name = getGameNameFromFile( file );
				}
			}
			else
			{
				name = getGameNameFromFile( file );
			}
		}
		else
		{
			name = getGameNameFromFile( file );
		}

		return name;
	}
	
	private String getGameNameFromFile( File file )
	{
		return file.getName().substring( 0, file.getName().lastIndexOf( '.' ) );
	}
}
