package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.extension.ExtensionLister;
import info.msxlaunchers.openmsx.game.repository.RepositoryData;
import info.msxlaunchers.openmsx.game.scan.Scanner;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.data.global.Settings;
import info.msxlaunchers.openmsx.launcher.data.global.constants.Language;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataGetter;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.settings.SettingsPersister;
import info.msxlaunchers.openmsx.launcher.starter.EmulatorStarter;
import info.msxlaunchers.openmsx.launcher.ui.presenter.GameLabel;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherException;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenterImpl;
import info.msxlaunchers.openmsx.launcher.ui.view.LauncherView;
import info.msxlaunchers.openmsx.machine.MachineLister;
import info.msxlaunchers.platform.FileLocator;

import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;


import org.junit.Before;
import org.junit.Test;

import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anySetOf;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.verify;

public class LauncherPresenterImplTest
{
	@Mock LauncherView view;
	@Mock SettingsPersister settingsPersister;
	@Mock GamePersister gamePersister;
	@Mock EmulatorStarter emulatorStarter;
	@Mock Scanner scanner;
	@Mock MachineLister machineLister;
	@Mock ExtensionLister extensionLister;
	@Mock ExtraDataGetter extraDataGetter;
	@Mock RepositoryData repositoryData;
	@Mock FileLocator fileLocator;

	private final String defaultDatabase = "defaultDatabase";

	@Before
	public void prepare()
	{
		MockitoAnnotations.initMocks( this );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg1() throws IOException
	{
		new LauncherPresenterImpl( null, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg2() throws IOException
	{
		new LauncherPresenterImpl( view, null, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg3() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, null, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg4() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, null, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg5() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, null, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg6() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, null, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg7() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, null, extraDataGetter, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg8() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, null, repositoryData, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg9() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, null, fileLocator );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorNullArg10() throws IOException
	{
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, null );
	}

	@Test( expected = IOException.class )
	public void testWhenSettingsPersisterThrowsException() throws IOException
	{
		when( settingsPersister.getSettings() ).thenThrow( new IOException() );

		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test
	public void testDefaultDatabaseNullInConstructor() throws IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, null, null ) );

		//no exception is thrown
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test
	public void testDatabaseNotFoundWhenRetrieveGamesInConstructor() throws DatabaseNotFoundException, IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( defaultDatabase ) ).thenThrow( new DatabaseNotFoundException( defaultDatabase ) );

		//no exception is thrown
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test
	public void testIOExceptionWhenRetrieveGamesInConstructor() throws DatabaseNotFoundException, IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( defaultDatabase ) ).thenThrow( new IOException() );

		//no exception is thrown
		new LauncherPresenterImpl( view, settingsPersister, gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );
	}

	@Test
	public void testStart() throws IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.start();
	}

	@Test
	public void testOnClickMenuOptionsSettings() throws IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsScreen();
	}

	@Test
	public void testOnClickMenuOptionsSettingsOKSettingsPersisted() throws IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, null );

		verify( settingsPersister, times( 1 ) ).saveSettings( new Settings( null, null, null, null, null ) );
	}

	@Test( expected = LauncherException.class )
	public void testOnClickMenuOptionsSettingsOKCannotSaveSettingsException() throws IOException, LauncherException
	{
		Settings settings = new Settings( null, null, null, defaultDatabase, null );
		when( settingsPersister.getSettings() ).thenReturn( settings );
		Mockito.doThrow( new IOException() ).when( settingsPersister ).saveSettings( settings );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, defaultDatabase, null );
	}

	@Test
	public void testOnClickMenuOptionsSettingsOKRefreshLanguage() throws IOException, LauncherException
	{
		LauncherPresenterImpl launcherPresenterImpl = null;
		String oldLanguageString = "ENGLISH";
		String newLanguageString = "SPANISH";
		Language oldLanguage = Language.valueOf( oldLanguageString );
		Language newLanguage = Language.valueOf( newLanguageString );

		//refresh language condition #1: old language is null and new language is not null
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, oldLanguageString );

		verify( view, times( 1 ) ).refreshLanguage( oldLanguage );
		
		//refresh language condition #2: old language is not null and new language is null
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, oldLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, null );

		//the following verification will only work on a machine where system default language is English
		verify( view, times( 2 ) ).refreshLanguage( oldLanguage );
		
		//refresh language condition #3: old and new languages are different
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, oldLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, newLanguageString );

		verify( view, times( 1 ) ).refreshLanguage( newLanguage );
	}

	@Test
	public void testOnClickMenuOptionsSettingsOKFlipViewOrientation() throws IOException, LauncherException
	{
		LauncherPresenterImpl launcherPresenterImpl = null;
		String enLanguageString = "ENGLISH";
		String esLanguageString = "SPANISH";
		String arLanguageString = "ARABIC";
		String faLanguageString = "PERSIAN";
		Language enLanguage = Language.valueOf( enLanguageString );
		Language arLanguage = Language.valueOf( arLanguageString );

		//flip orientation condition #1: old language is left-to-right and new language is left-to-right
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, enLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, esLanguageString );

		verify( view, times( 0 ) ).flipOrientationLeftToRight();
		verify( view, times( 0 ) ).flipOrientationRightToLeft();

		//flip orientation condition #2: old language is right-to-left and new language is right-to-left
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, arLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, faLanguageString );

		verify( view, times( 0 ) ).flipOrientationLeftToRight();
		verify( view, times( 0 ) ).flipOrientationRightToLeft();

		//flip orientation condition #3: old language is left-to-right and new language is right-to-left
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, enLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, faLanguageString );

		verify( view, times( 0 ) ).flipOrientationLeftToRight();
		verify( view, times( 1 ) ).flipOrientationRightToLeft();

		//flip orientation condition #4: old language is right-to-left and new language is left-to-right
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, arLanguage ) );

		launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister, gamePersister,
				emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onRequestSettingsAction( null, null, null, null, enLanguageString );

		verify( view, times( 1 ) ).flipOrientationLeftToRight();
		verify( view, times( 1 ) ).flipOrientationRightToLeft();
	}

	@Test( expected = LauncherException.class )
	public void testOnSelectDatabaseIOExceptionWhenRetrieveGames()
			throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( newDatabase ) ).thenThrow( new IOException() );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onSelectDatabase( newDatabase );
	}

	@Test( expected = LauncherException.class )
	public void testOnSelectDatabaseDatabaseNotFoundExceptionWhenRetrieveGames()
			throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( newDatabase ) ).thenThrow( new DatabaseNotFoundException( newDatabase ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onSelectDatabase( newDatabase );
	}

	@Test
	public void testOnSelectDatabaseDatabase() throws IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onSelectDatabase( newDatabase );

		verify( view, times( 1 ) ).fillGameList( newDatabase, new LinkedHashSet<GameLabel>(), null );
		verify( view, times( 1 ) ).updateGameCount( 0 );
	}

	@Test
	public void testOnSelectDatabaseDatabaseNullOrSameDatabase() throws IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		//case 1: null database
		launcherPresenterImpl.onSelectDatabase( null );

		verify( view, never() ).fillGameList( null, new LinkedHashSet<GameLabel>(), null );
		verify( view, never() ).updateGameCount( 0 );

		//case 2: same database
		launcherPresenterImpl.onSelectDatabase( defaultDatabase );

		verify( view, never() ).fillGameList( defaultDatabase, new LinkedHashSet<GameLabel>(), null );
		verify( view, never() ).updateGameCount( 0 );
	}

	@Test
	public void testOnStartEmulatorFromGameList()
			throws IOException, LauncherException
	{
		Settings settings = new Settings( null, null, null, defaultDatabase, null );

		when( settingsPersister.getSettings() ).thenReturn( settings );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onLaunchGame( "gameName" );
	}

	@Test( expected = LauncherException.class )
	public void testOnStartEmulatorFromGameListExceptionWhenStartEmulator()
			throws IOException, LauncherException
	{
		Settings settings = new Settings( null, null, null, defaultDatabase, null );

		when( emulatorStarter.start( (Settings)any(), (Game)any() ) ).thenThrow( new IOException() );
		when( settingsPersister.getSettings() ).thenReturn( settings );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		launcherPresenterImpl.onLaunchGame( "gameName" );
	}

	@Test( expected = LauncherException.class )
	public void testOnRemoveGamesIOException()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		doThrow( new IOException() ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		Set<String> gameNames = new HashSet<String>();
		launcherPresenterImpl.onRequestRemoveGamesAction( gameNames, defaultDatabase );
	}

	@Test( expected = LauncherException.class )
	public void testOnRemoveGamesDatabaseDatabaseNotFoundException()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		doThrow( new DatabaseNotFoundException( defaultDatabase ) ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		Set<String> gameNames = new HashSet<String>();
		launcherPresenterImpl.onRequestRemoveGamesAction( gameNames, defaultDatabase );
	}

	@Test
	public void testOnRemoveNonExistentGamesDatabase()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		Set<String> gameNames = new HashSet<String>();
		gameNames.add( "gameName" );

		doThrow( new GameNotFoundException( "gameName" ) ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		launcherPresenterImpl.onRequestRemoveGamesAction( gameNames, defaultDatabase );

		verify( view, times( 1 ) ).updateGameCount( 0 );
	}

	@Test
	public void testOnRemoveGamesDatabase()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		LauncherPresenterImpl launcherPresenterImpl = new LauncherPresenterImpl( view, settingsPersister,
				gamePersister, emulatorStarter, scanner, machineLister, extensionLister, extraDataGetter, repositoryData, fileLocator );

		Set<String> gameNames = new HashSet<String>();
		gameNames.add( "gameName" );

		launcherPresenterImpl.onRequestRemoveGamesAction( gameNames, defaultDatabase );

		verify( view, times( 1 ) ).updateGameCount( 0 );
	}
}