/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.view.swing;

import info.msxlaunchers.openmsx.common.Utils;
import info.msxlaunchers.openmsx.launcher.data.global.constants.Language;
import info.msxlaunchers.openmsx.launcher.ui.presenter.GameLabel;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherException;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter;
import info.msxlaunchers.openmsx.launcher.ui.presenter.Medium;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.component.BackgroundPanel;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.component.JListWithImagesAndActions;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.component.MessageBoxUtil;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.language.LanguageDisplayFactory;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Main window class
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
@SuppressWarnings("serial")
public class MainWindow extends JFrame implements ActionListener, WindowFocusListener
{
	private final LauncherPresenter presenter;

	private Set<String> databases;
	private String currentDatabase;

	private JPanel contentPane;
	private JMenuBar menuBar;
	private JMenu optionsMenu;
	private JMenuItem optionsSettings;
	private JMenu actionsMenu;
	private JMenuItem actionsFillDatabase;
	private JMenuItem actionsCreateEmptyDatabase;
	private JMenu helpMenu;
	private JMenuItem helpAbout;
	private JMenuItem helpFile;
	private JListWithImagesAndActions gameList;
	private JLabel databaseLabel;
	private JLabel countLabel;
	private JLabel countValueLabel;
	private JLabel gameDataLabel;
	private JComboBox databaseComboBox;
	private DefaultComboBoxModel databaseComboBoxModel;
	private JButton launchButton;
	private JButton removeButton;
	private JButton addButton;
	private JButton editButton;
	private JButton infoButton;

	private JPopupMenu contextMenu;
	private JMenuItem moveMenuItem;
	private JMenuItem locateFileMenuItem;

	private Map<String,String> messages;

	private final ImageIcon noScreenshotImage = new ImageIcon(getClass().getResource("images/noscrsht.png"));
	private JLabel screenshot1Label;
	private JLabel screenshot2Label;

	private String confirmationTitle = null;
	private String removeConfirmationMessage = null;

	private final String MAIN_ICON = "images/icon-32.png";
	private final String MEDIA_DIRECTORY = "images/media/";
	private Map<Medium, ImageIcon> mediaIconsMap = new HashMap<Medium,ImageIcon>();

	public static final Dimension buttonDimension = new Dimension(109, 28);
	private static final Font FONT_SIZE_11 = new Font(null, Font.PLAIN, 11);
	private static final Font FONT_SIZE_10 = new Font(null, Font.PLAIN, 10);

	public MainWindow(LauncherPresenter presenter)
	{
		this.presenter = presenter;
		try
		{
		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels())
		    {
		        if ("Nimbus".equals(info.getName()))
		        {
		            UIManager.setLookAndFeel(info.getClassName());
		            break;
		        }
		    }
		}
		catch (Exception e)
		{
		    //should not happen
		}

		setIconImage(new ImageIcon(getClass().getResource(MAIN_ICON)).getImage());

		//Initialise media icons map
		mediaIconsMap.put( Medium.ROM, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "rom.png")));
		mediaIconsMap.put( Medium.DISK, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "disk.png")));
		mediaIconsMap.put( Medium.TAPE, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "tape.png")));
		mediaIconsMap.put( Medium.HARDDISK, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "harddisk.png")));
		mediaIconsMap.put( Medium.LASERDISC, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "laserdisc.png")));
		mediaIconsMap.put( Medium.SCRIPT, new ImageIcon(getClass().getResource(MEDIA_DIRECTORY + "script.png")));
	}

	public void display(Language language,
			Set<GameLabel> games,
			Set<String> databases,
			String currentDatabase,
			boolean rightToLeft)
	{
		this.databases = databases;
		this.currentDatabase = currentDatabase;

        ImageIcon imageIcon = new ImageIcon(getClass().getResource("images/bkgrd.png"));
		contentPane = new BackgroundPanel(imageIcon.getImage(), BackgroundPanel.ACTUAL);
		setContentPane(contentPane);
		contentPane.setLayout(null);
        setTitle("openMSX Launcher");
		setSize(728, 605);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setResizable(false);

        //menus
//		drawMenu();

		//main window
		drawScreen();
		
		//set the orientation
		if(rightToLeft)
		{
			flipOrientationRightToLeft();
		}
		else
		{
			flipOrientationLeftToRight();
		}

		//setup context menu
		setupContextMenu();

		//set language labels
		refreshLanguage(language);

		//set the database and game list
		fillGameList(currentDatabase, games, null);

		//update the count
		updateGameCount(games.size());

		addWindowFocusListener(this);

		setVisible(true);
	}

	private void drawMenu()
	{
		final JFrame tmp = this;
		menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        //options menu
		optionsMenu = new JMenu();
//		optionsMenu.setMnemonic(KeyEvent.VK_O);
		menuBar.add(optionsMenu);

		optionsSettings = new JMenuItem();
//        optionsSettings.setMnemonic(KeyEvent.VK_S);

        optionsSettings.addActionListener(new ActionListener() {
        	@Override
            public void actionPerformed(ActionEvent event) {
                presenter.onRequestSettingsScreen();
            }
        });

        optionsMenu.add(optionsSettings);

        //actions menu
		actionsMenu = new JMenu();
		menuBar.add(actionsMenu);

        actionsCreateEmptyDatabase = new JMenuItem();
        
        actionsCreateEmptyDatabase.addActionListener(new ActionListener() {
        	@Override
        	public void actionPerformed(ActionEvent event) {
        		presenter.onRequestCreateEmptyDatabaseScreen();
        	}
        });

        actionsMenu.add(actionsCreateEmptyDatabase);

        actionsFillDatabase = new JMenuItem();
      
        actionsFillDatabase.addActionListener(new ActionListener() {
        	@Override
        	public void actionPerformed(ActionEvent event) {
        		try
        		{
        			presenter.onRequestFillDatabaseScreen();
				}
        		catch(LauncherException le)
        		{
					MessageBoxUtil.showErrorMessageBox(tmp, le, messages);
				}
        	}
        });

        actionsMenu.add(actionsFillDatabase);

        //help menu
        helpMenu = new JMenu();
        menuBar.add(helpMenu);

        helpFile = new JMenuItem();
        
        helpFile.addActionListener(new ActionListener() {
        	@Override
        	public void actionPerformed(ActionEvent event) {
        		try
        		{
        			presenter.onRequestHelpFile();
        		}
        		catch(LauncherException le)
        		{
        			MessageBoxUtil.showErrorMessageBox(tmp, le, messages);
        		}
        	}
        });

        helpMenu.add(helpFile);

        helpMenu.addSeparator();

        helpAbout = new JMenuItem();
        
        helpAbout.addActionListener(new ActionListener() {
        	@Override
        	public void actionPerformed(ActionEvent event) {
        		presenter.onRequestAboutScreen();
        	}
        });

        helpMenu.add(helpAbout);
	}

	public void addDatabase(String database)
	{
		databaseComboBoxModel.addElement(database);
	}

	private void drawScreen()
	{
		final JFrame tmp = this;
		databaseLabel = new JLabel();
		databaseLabel.setFont(FONT_SIZE_11);

		databaseComboBoxModel = new SortedComboBoxModel(Utils.getSortedCaseInsensitiveArray(databases));
		databaseComboBox = new JComboBox(databaseComboBoxModel);
		databaseComboBox.setMaximumRowCount(10);
		databaseComboBox.setFont(FONT_SIZE_11);

		databaseComboBox.addActionListener (new ActionListener () {
		    public void actionPerformed(ActionEvent e) {
		    	try
		    	{
		    		presenter.onSelectDatabase((String)databaseComboBox.getSelectedItem());
				}
				catch(LauncherException le)
				{
					//here a non-existing database was selected.
					//In this case force selection of previous valid one
					databaseComboBox.setSelectedItem(currentDatabase);

					MessageBoxUtil.showErrorMessageBox(tmp, le, messages);
				}

		    	currentDatabase = (String)databaseComboBox.getSelectedItem();
		    }
		});

		countLabel = new JLabel();
		countLabel.setFont(FONT_SIZE_11);
		countLabel.setHorizontalAlignment(SwingConstants.CENTER);

		countValueLabel = new JLabel();
		countValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
		countValueLabel.setFont(FONT_SIZE_11);

		gameDataLabel = new JLabel();
		gameDataLabel.setFont(FONT_SIZE_10);
		gameDataLabel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.GRAY, 1),
				BorderFactory.createEmptyBorder(0, 3, 0, 3)));
		gameDataLabel.setOpaque(true);

		gameList = new JListWithImagesAndActions(new DefaultListModel());
		JScrollPane gameListScrollBar = new JScrollPane(gameList);

		JPanel launchButtonPanel = new JPanel();

		JPanel removeButtonPanel = new JPanel();
		removeButtonPanel.setLayout(new GridLayout(1, 0, 0, 0));

		JPanel addButtonPanel = new JPanel();
		addButtonPanel.setLayout(new GridLayout(1, 0, 0, 0));

		JPanel editButtonPanel = new JPanel();
		editButtonPanel.setLayout(new GridLayout(1, 0, 0, 0));

		JPanel infoButtonPanel = new JPanel();
		infoButtonPanel.setLayout(new GridLayout(1, 0, 0, 0));

		screenshot1Label = new JLabel(noScreenshotImage);

		screenshot2Label = new JLabel(noScreenshotImage);

		GroupLayout groupLayout = new GroupLayout(getContentPane());
		groupLayout.setHorizontalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addGap(21)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(gameListScrollBar, GroupLayout.PREFERRED_SIZE, 270, GroupLayout.PREFERRED_SIZE)
						.addComponent(gameDataLabel, GroupLayout.PREFERRED_SIZE, 270, GroupLayout.PREFERRED_SIZE)
						.addGroup(groupLayout.createSequentialGroup()
							.addComponent(databaseComboBox, GroupLayout.PREFERRED_SIZE, 178, GroupLayout.PREFERRED_SIZE)
							.addGap(30)
							.addComponent(false, countValueLabel, 30, GroupLayout.DEFAULT_SIZE, 30))
						.addGroup(groupLayout.createSequentialGroup()
							.addComponent(false, databaseLabel, 100, GroupLayout.DEFAULT_SIZE, 100)
							.addGap(94)
							.addComponent(false,countLabel, 60, GroupLayout.DEFAULT_SIZE, 60)))
					.addGap(18)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(launchButtonPanel, GroupLayout.PREFERRED_SIZE, 103, GroupLayout.PREFERRED_SIZE)
						.addComponent(removeButtonPanel, GroupLayout.PREFERRED_SIZE, 103, GroupLayout.PREFERRED_SIZE)
						.addComponent(addButtonPanel, GroupLayout.PREFERRED_SIZE, 103, GroupLayout.PREFERRED_SIZE)
						.addComponent(editButtonPanel, GroupLayout.PREFERRED_SIZE, 103, GroupLayout.PREFERRED_SIZE)
						.addComponent(infoButtonPanel, GroupLayout.PREFERRED_SIZE, 103, GroupLayout.PREFERRED_SIZE))
					.addGap(18)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(screenshot1Label, GroupLayout.PREFERRED_SIZE, 272, GroupLayout.PREFERRED_SIZE)
						.addComponent(screenshot2Label, GroupLayout.PREFERRED_SIZE, 272, GroupLayout.PREFERRED_SIZE))
					.addGap(35))
		);
		groupLayout.setVerticalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addGap(15)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addGroup(groupLayout.createSequentialGroup()
							.addGroup(groupLayout.createParallelGroup(Alignment.BASELINE)
								.addComponent(databaseLabel)
								.addComponent(countLabel))
							.addGroup(groupLayout.createParallelGroup(Alignment.BASELINE)
								.addComponent(databaseComboBox, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(countValueLabel))
							.addGap(4)
							.addComponent(gameDataLabel, GroupLayout.PREFERRED_SIZE, 14, GroupLayout.PREFERRED_SIZE)
							.addGap(4)
								.addComponent(gameListScrollBar, GroupLayout.PREFERRED_SIZE, 448, GroupLayout.PREFERRED_SIZE))
							.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
								.addGroup(groupLayout.createSequentialGroup()
									.addGap(160)
									.addComponent(launchButtonPanel, GroupLayout.PREFERRED_SIZE, 29, GroupLayout.PREFERRED_SIZE)
									.addPreferredGap(ComponentPlacement.RELATED)
									.addComponent(removeButtonPanel, GroupLayout.PREFERRED_SIZE, 29, GroupLayout.PREFERRED_SIZE)
									.addPreferredGap(ComponentPlacement.RELATED)
									.addComponent(addButtonPanel, GroupLayout.PREFERRED_SIZE, 29, GroupLayout.PREFERRED_SIZE)
									.addPreferredGap(ComponentPlacement.RELATED)
									.addComponent(editButtonPanel, GroupLayout.PREFERRED_SIZE, 29, GroupLayout.PREFERRED_SIZE)
									.addPreferredGap(ComponentPlacement.RELATED)
									.addComponent(infoButtonPanel, GroupLayout.PREFERRED_SIZE, 29, GroupLayout.PREFERRED_SIZE))
								.addGroup(groupLayout.createSequentialGroup()
									.addGap(15)
									.addComponent(screenshot1Label, GroupLayout.PREFERRED_SIZE, 240, GroupLayout.PREFERRED_SIZE)
									.addGap(15)
									.addComponent(screenshot2Label, GroupLayout.PREFERRED_SIZE, 240, GroupLayout.PREFERRED_SIZE)))))
		);
		
		infoButton = new JButton();
		infoButton.addActionListener(this);
		infoButtonPanel.add(infoButton);
		
		editButton = new JButton();
		editButton.addActionListener(this);
		editButtonPanel.add(editButton);
		
		addButton = new JButton();
		addButton.addActionListener(this);
		addButtonPanel.add(addButton);
		
		removeButton = new JButton();
		removeButton.addActionListener(this);
		removeButtonPanel.add(removeButton);
		launchButtonPanel.setLayout(new GridLayout(1, 0, 0, 0));
		
		launchButton = new JButton();
		launchButton.addActionListener(this);
		launchButtonPanel.add(launchButton);
		getContentPane().setLayout(groupLayout);

	    ListSelectionListener listSelectionListener = new ListSelectionListener()
	    {
			@Override
	    	public void valueChanged(ListSelectionEvent listSelectionEvent)
	    	{
	    		if(!listSelectionEvent.getValueIsAdjusting())
	    		{
	    			presenter.onSelectGames(getSelectedGames());
	    		}
	    	}
	    };
	    gameList.addListSelectionListener(listSelectionListener);

	    ActionListener actionListener = new ActionListener()
	    {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				if(e.getActionCommand().equals(JListWithImagesAndActions.DOUBLE_CLICK_COMMAND) ||
						e.getActionCommand().equals(JListWithImagesAndActions.ENTER_KEY_COMMAND))
				{
					presenter.onSelectGames(getSelectedGames());
					launchGame();
				}
				else if(e.getActionCommand().equals(JListWithImagesAndActions.DELETE_KEY_COMMAND))
				{
					promptForRemovingSelection();
				}
			}
		};
		gameList.addActionListener(actionListener);

		MouseAdapter mouseAdapter = new MouseAdapter()
		{
			public void mouseClicked(MouseEvent me)
			{
				 if(SwingUtilities.isRightMouseButton(me))
				 {
					 int mouseSelectionIndex = gameList.locationToIndex(me.getPoint());
					 if(!isInSelection(gameList.getSelectedIndices(), mouseSelectionIndex))
					 {
						 gameList.setSelectedIndex(mouseSelectionIndex);
					 }
					 //need to disable the move menu item if there was only one database
					 moveMenuItem.setEnabled(databases.size() > 1);

					 //need to disable the locate file menu item if more than one game was selected
					 locateFileMenuItem.setEnabled(gameList.getSelectedIndices().length == 1);

					 contextMenu.show(gameList, me.getX(), me.getY());
				 }
			}
		};
		gameList.addMouseListener(mouseAdapter);

	}

	public void flipOrientationLeftToRight()
	{
		flipOrientation(ComponentOrientation.LEFT_TO_RIGHT);
	}

	public void flipOrientationRightToLeft()
	{
		flipOrientation(ComponentOrientation.RIGHT_TO_LEFT);
	}

	public void refreshLanguage(Language language)
	{
		messages = LanguageDisplayFactory.getDisplayMessages(getClass(), language);

		optionsMenu.setText(messages.get("OPTIONS"));
		optionsSettings.setText(messages.get("SETTINGS") + "...");
		actionsMenu.setText(messages.get("ACTIONS"));
		actionsCreateEmptyDatabase.setText(messages.get("CREATE_EMPTY_DATABASE") + "...");
		actionsFillDatabase.setText(messages.get("FILL_DATABASE") + "...");
		helpMenu.setText(messages.get("HELP"));
		helpFile.setText(messages.get("HELP"));
		helpAbout.setText(messages.get("ABOUT"));
		databaseLabel.setText(messages.get("DATABASE"));
		countLabel.setText(messages.get("COUNT"));
		launchButton.setText(messages.get("LAUNCH"));
		removeButton.setText(messages.get("REMOVE"));
		addButton.setText(messages.get("ADD"));
		editButton.setText(messages.get("EDIT"));
		infoButton.setText(messages.get("INFO"));

		moveMenuItem.setText(messages.get("MOVE") + "...");
		locateFileMenuItem.setText(messages.get("LOCATE_FILE"));

		confirmationTitle = messages.get("CONFIRMATION");
		removeConfirmationMessage = messages.get("REMOVE_CONFIRMATION_MESSAGE");
	}

	public void fillGameList(String currentDatabase, Set<GameLabel> games, String selectedGame)
	{
		databaseComboBox.setSelectedItem(currentDatabase);
		gameList.clear();

		for(GameLabel gameLabel: games)
		{
			gameList.addElement(gameLabel.getName(), mediaIconsMap.get(gameLabel.getMedium()));
		}

		if(selectedGame != null)
		{
			gameList.setSelectedValue(selectedGame);
		}
		else
		{
			enableButtons(false, false, false, false);
		}
	}

	public void updateGameCount(int total)
	{
		countValueLabel.setText(String.valueOf(total));
	}

	/*
	 * screenshot1: string pointing to Screenshot1 - if null both screenshots will show noscreenshot
	 * screenshot2: string pointing to Screenshot2 - if null screenshot2 will show noscreenshot
	 */
	public void showGameScreenshot(String screenshot1, String screenshot2)
	{
		if(screenshot1 == null)
		{
			screenshot1Label.setIcon(noScreenshotImage);
			screenshot2Label.setIcon(noScreenshotImage);
		}
		else
		{
			screenshot1Label.setIcon(new ImageIcon(screenshot1));

			if(screenshot2 == null)
			{
				screenshot2Label.setIcon(noScreenshotImage);
			}
			else
			{
				screenshot2Label.setIcon(new ImageIcon(screenshot2));
			}
		}
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == launchButton)
		{
			launchGame();
		}
		if(e.getSource() == addButton)
		{
			addGame();
		}
		if(e.getSource() == editButton)
		{
			editGame();
		}
		else if(e.getSource() == removeButton)
		{
			promptForRemovingSelection();
		}
		else if(e.getSource() == infoButton)
		{
			viewGameInfo(getSelectedGame());
		}
		else if(e.getSource() == moveMenuItem)
		{
			moveSelectedGames(getSelectedGames());
		}
		else if(e.getSource() == locateFileMenuItem)
		{
			locateMainFileOfSelectedGame(getSelectedGame());
		}
	}

	public void showGameCompanyYearSizeData(String company, String year, long size)
	{
		String displayString;
		if(Utils.isEmpty(company) && Utils.isEmpty(year))
		{
			if(size == 0.0)
			{
				//size 0 is for files with real size of 0, files that don't exist and scripts
				displayString = "";
			}
			else
			{
				displayString = size + " KB";
			}
		}
		else
		{
			displayString = company + " " + year + " - " + size + " KB";
		}
		gameDataLabel.setText(displayString);
	}

	public void resetGameCompanyYearSizeData()
	{
		gameDataLabel.setText("");
	}

	@Override
	public void windowGainedFocus(WindowEvent arg0)
	{
		gameList.requestFocusInWindow();
	}

	@Override
	public void windowLostFocus(WindowEvent arg0)
	{
		//just ignore
	}
	
	public void enableButtons( boolean launchFlag, boolean removeFlag, boolean editFlag, boolean infoFlag )
	{
		launchButton.setEnabled(launchFlag);
		removeButton.setEnabled(removeFlag);
		editButton.setEnabled(editFlag);
		infoButton.setEnabled(infoFlag);		
	}

	private void launchGame()
	{
		try
		{
			presenter.onLaunchGame(getSelectedGame());
		}
		catch(LauncherException le)
		{
			MessageBoxUtil.showErrorMessageBox(this, le, messages);
		}
	}

	private void addGame()
	{
		try
		{
			presenter.onRequestAddGameScreen();
		}
		catch(LauncherException le)
		{
			MessageBoxUtil.showErrorMessageBox(this, le, messages);
		}
	}

	private void editGame()
	{
		try
		{
			presenter.onRequestEditGameScreen(getSelectedGame());
		}
		catch(LauncherException le)
		{
			MessageBoxUtil.showErrorMessageBox(this, le, messages);
		}
	}

	private void promptForRemovingSelection()
	{
		if(JOptionPane.showConfirmDialog(this, removeConfirmationMessage, confirmationTitle, JOptionPane.YES_NO_OPTION) == 0)
		{
			try
			{
				presenter.onRequestRemoveGamesAction(getSelectedGames(), currentDatabase);

				//delete from the Jlist - start with higher indices then go down
				//the reason for this is that deleting lower indices first will shift elements up so indices of later items will be different
		        int selections[] = gameList.getSelectedIndices();
		        for(int index=selections.length-1; index >= 0; index--)
		        {
		        	gameList.remove(selections[index]);
		        }
			}
			catch(LauncherException le)
			{
				MessageBoxUtil.showErrorMessageBox(this, le, messages);
			}
		}
	}

	private void viewGameInfo(String gameInfo)
	{
		try
		{
			presenter.onRequestGameInfo(gameInfo);
		}
		catch(LauncherException le)
		{
			MessageBoxUtil.showErrorMessageBox(this, le, messages);
		}
	}

	private void moveSelectedGames(Set<String> games)
	{
		Set<String> movedGames = presenter.onRequestMoveGamesScreen(games, (String)databaseComboBox.getSelectedItem());

		int gamesTotal = gameList.getListSize();
		for(int index = gamesTotal - 1; index > -1; index--)
		{
			if(movedGames.contains(gameList.getElementAt(index)))
			{
				gameList.remove(index);
			}
		}
	}

	private void locateMainFileOfSelectedGame(String game)
	{
		try
		{
			presenter.onRequestLocateFile(getSelectedGame());
		}
		catch(LauncherException le)
		{
			MessageBoxUtil.showErrorMessageBox(this, le, messages);
		}
	}

	private boolean isInSelection(int[] selections, int mouseSelection)
	{
		for(int ix = 0; ix < selections.length; ix++)
		{
			if(selections[ix] == mouseSelection)
			{
				return true;
			}
		}
		return false;
	}

	private void flipOrientation(ComponentOrientation co)
	{
		//I had to put this here because of the alignment of the Arabic menu when changing the language in Settings
		drawMenu();

		menuBar.setComponentOrientation(co);
		optionsMenu.applyComponentOrientation(co);
		actionsMenu.applyComponentOrientation(co);
		helpMenu.applyComponentOrientation(co);
		contentPane.setComponentOrientation(co);
		contentPane.applyComponentOrientation(co);

		//force the following to always be left to right
		gameList.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		databaseComboBox.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
	}

	private String getSelectedGame()
	{
		String gameName = null;
		Set<String> selectedGames = getSelectedGames();

		if(selectedGames.size() == 1)
		{
			gameName = selectedGames.iterator().next();
		}

		return gameName;
	}

	private Set<String> getSelectedGames()
	{
		Set<String> gameNames = null;
        int selections[] = gameList.getSelectedIndices();
        if(selections != null && selections.length > 0)
        {
	        gameNames = new HashSet<String>(Arrays.asList(gameList.getSelectedItems()));
        }
        return gameNames;
	}

	private void setupContextMenu()
	{
	    contextMenu = new JPopupMenu();

	    moveMenuItem = new JMenuItem();
	    moveMenuItem.addActionListener(this);
	    contextMenu.add(moveMenuItem);

	    locateFileMenuItem = new JMenuItem();
	    locateFileMenuItem.addActionListener(this);
	    contextMenu.add(locateFileMenuItem);
	}

	private class SortedComboBoxModel extends DefaultComboBoxModel
	{
		public SortedComboBoxModel(String[] elements)
		{
			super(elements);
		}

		@Override
        public void addElement(Object element)
		{
			int index;
			int size = getSize();

			for(index = 0; index < size; index++)
			{
				if(((String)element).compareToIgnoreCase((String)getElementAt(index)) < 0)
				{
					break;
				}
			}

			insertElementAt(element, index);
        }
	}
}
